import { AutomizerParams, AutomizerSummary, IPresentationProps, PresTemplate, RootPresTemplate } from './definitions/app';
export default class Automizer implements IPresentationProps {
    rootTemplate: RootPresTemplate;
    templates: PresTemplate[];
    templateDir: string;
    outputDir: string;
    timer: number;
    params: AutomizerParams;
    /**
     * Parameters for Automizer constructor.
     * @param {string} templateDir - optional: prefix for all templates to load.
     * @param {string} outputDir - optional: prefix for all output files.
     */
    constructor(params?: AutomizerParams);
    /**
     * Load a pptx file and set it as root template.
     * @param {string} location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @return {Automizer} Instance of Automizer
     */
    loadRoot(location: string): this;
    /**
     * Load a template pptx file.
     * @param {string} location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @param {string} name - Optional: A short name for the template. If skipped, the template will be named by its location.
     * @return {Automizer} Instance of Automizer
     */
    load(location: string, name?: string): this;
    loadTemplate(location: string, name?: string): this;
    isPresTemplate(template: PresTemplate | RootPresTemplate): template is PresTemplate;
    /**
     * Find imported template by given name and return a certain slide by number.
     * @param {string} name - Name of template; must be imported by Automizer.importTemplate()
     * @param {number} slideNumber - Number of slide in template presentation
     * @return {Automizer} Instance of Automizer
     */
    addSlide(name: string, slideNumber: number, callback?: Function): this;
    template(name: string): PresTemplate;
    getLocation(location: string, type?: string): string;
    write(location: string): Promise<AutomizerSummary>;
}
