/// <reference types="node" />
import JSZip from "jszip";
import { ElementType } from "./enums";
export declare type AutomizerParams = {
    templateDir?: string;
    outputDir?: string;
};
export declare type AutomizerSummary = {
    status: string;
    duration: number;
    file: string;
    templates: number;
    slides: number;
    charts: number;
    images: number;
};
export interface ICounter {
    set(): void | PromiseLike<void>;
    get(): number;
    name: string;
    count: number;
    _increment(): number;
}
export interface ISlide {
    append(targetTemplate: RootPresTemplate): Promise<void>;
    addElement(presName: string, slideNumber: number, selector: Function | string): void;
    sourceArchive: JSZip;
    sourceNumber: number;
    modifications: Function[];
    modify: Function;
}
export interface IPresentationProps {
    rootTemplate: RootPresTemplate;
    templates: PresTemplate[];
    template(name: string): PresTemplate;
    params: AutomizerParams;
    timer: number;
}
export interface ITemplate {
    location: string;
    file: Promise<Buffer>;
    archive: Promise<JSZip>;
}
export interface RootPresTemplate extends ITemplate {
    slides: ISlide[];
    counter: ICounter[];
    count(name: string): number;
    incrementCounter(name: string): number;
    appendSlide(slide: ISlide): Promise<void>;
}
export interface PresTemplate extends ITemplate {
    name: string;
}
export interface IShape {
    sourceArchive: JSZip;
    targetArchive: JSZip;
}
export interface IChart extends IShape {
    append: Function;
    sourceNumber: number;
    targetNumber: number;
}
export interface IImage extends IShape {
    append: Function;
    sourceFile: string;
    targetFile: string;
    contentTypeMap: any;
}
export declare type Target = {
    file: string;
    number: number;
    rId?: string;
};
export declare type ImportElement = {
    presName: string;
    slideNumber: number;
    selector: string;
    mode: string;
    callback?: Function | Function[];
};
export declare type ImportedElement = {
    mode: string;
    name?: string;
    sourceArchive: JSZip;
    sourceSlideNumber: number;
    callback?: any;
    target?: AnalyzedElementType["target"];
    type?: AnalyzedElementType["type"];
    sourceElement?: AnalyzedElementType["element"];
};
export declare type AnalyzedElementType = {
    type: ElementType;
    target?: Target;
    element?: HTMLElement;
};
export declare type TargetByRelIdMapParam = {
    relRootTag: string;
    relAttribute: string;
    prefix: string;
    expression?: RegExp;
};
export declare type Workbook = {
    archive: JSZip;
    sheet: Document | any;
    sharedStrings: Document;
    table: Document;
};
