"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xml_1 = __importDefault(require("./xml"));
class CountHelper {
    constructor(name, template) {
        this.name = name;
        this.template = template;
    }
    static increment(name, counters) {
        return CountHelper.getCounterByName(name, counters)._increment();
    }
    static count(name, counters) {
        return CountHelper.getCounterByName(name, counters).get();
    }
    static getCounterByName(name, counters) {
        let counter = counters.find(counter => counter.name === name);
        if (counter === undefined) {
            throw new Error(`Counter ${name} not found.`);
        }
        return counter;
    }
    _increment() {
        this.count++;
        return this.count;
    }
    set() {
        return __awaiter(this, void 0, void 0, function* () {
            let method = this.getCounterMethod();
            if (method === undefined) {
                throw new Error(`No way to count ${this.name}.`);
            }
            this.count = yield method(yield this.template.archive);
        });
    }
    get() {
        return this.count;
    }
    getCounterMethod() {
        switch (this.name) {
            case 'slides': return CountHelper.countSlides;
            case 'charts': return CountHelper.countCharts;
            case 'images': return CountHelper.countImages;
        }
    }
    static countSlides(presentation) {
        return __awaiter(this, void 0, void 0, function* () {
            let presentationXml = yield xml_1.default.getXmlFromArchive(presentation, 'ppt/presentation.xml');
            let slideCount = presentationXml.getElementsByTagName('p:sldId').length;
            return slideCount;
        });
    }
    static countCharts(presentation) {
        return __awaiter(this, void 0, void 0, function* () {
            let contentTypesXml = yield xml_1.default.getXmlFromArchive(presentation, '[Content_Types].xml');
            let overrides = contentTypesXml.getElementsByTagName('Override');
            let chartCount = 0;
            for (let i in overrides) {
                let override = overrides[i];
                if (override.getAttribute) {
                    let contentType = override.getAttribute('ContentType');
                    if (contentType === `application/vnd.openxmlformats-officedocument.drawingml.chart+xml`) {
                        chartCount++;
                    }
                }
            }
            return chartCount;
        });
    }
    static countImages(presentation) {
        return __awaiter(this, void 0, void 0, function* () {
            let files = presentation.file(/ppt\/media\/image/);
            return files.length;
        });
    }
}
exports.default = CountHelper;
//# sourceMappingURL=count.js.map