"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xmldom_1 = require("xmldom");
const file_1 = __importDefault(require("./file"));
const constants_1 = require("../definitions/constants");
const pretty_1 = __importDefault(require("./pretty"));
class XmlHelper {
    static getXmlFromArchive(archive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            let xmlDocument = yield file_1.default.extractFromArchive(archive, file);
            const dom = new xmldom_1.DOMParser();
            return dom.parseFromString(xmlDocument);
        });
    }
    static writeXmlToArchive(archive, file, xml) {
        return __awaiter(this, void 0, void 0, function* () {
            let s = new xmldom_1.XMLSerializer();
            let xmlBuffer = s.serializeToString(xml);
            return archive.file(file, xmlBuffer);
        });
    }
    static appendIf(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            if (element.clause !== undefined) {
                if (!element.clause(xml))
                    return false;
            }
            return XmlHelper.append(element);
        });
    }
    static append(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            let newElement = xml.createElement(element.tag);
            for (let attribute in element.attributes) {
                let value = element.attributes[attribute];
                newElement.setAttribute(attribute, (typeof value === 'function') ? value(xml) : value);
            }
            let parent = element.parent(xml);
            parent.appendChild(newElement);
            XmlHelper.writeXmlToArchive(element.archive, element.file, xml);
            return newElement;
        });
    }
    static getNextRelId(rootArchive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            let presentationRelsXml = yield XmlHelper.getXmlFromArchive(rootArchive, file);
            let increment = (max) => 'rId' + max;
            let rid = XmlHelper.getMaxId(presentationRelsXml.documentElement.childNodes, 'Id', true);
            return increment(rid);
        });
    }
    static getMaxId(rels, attribute, increment) {
        let max = 0;
        for (let i in rels) {
            let rel = rels[i];
            if (rel.getAttribute !== undefined) {
                let id = Number(rel.getAttribute(attribute).replace('rId', ''));
                max = (id > max) ? id : max;
            }
        }
        switch (typeof increment) {
            case 'boolean': return ++max;
            default: return max;
        }
    }
    static getTargetsFromRelationships(archive, path, prefix, suffix) {
        return __awaiter(this, void 0, void 0, function* () {
            return XmlHelper.getRelationships(archive, path, (element, rels) => {
                let target = element.getAttribute('Target');
                if (target.indexOf(prefix) === 0) {
                    rels.push({
                        file: target,
                        rId: element.getAttribute('Id'),
                        number: Number(target.replace(prefix, '').replace(suffix || '.xml', ''))
                    });
                }
            });
        });
    }
    static getTargetsByRelationshipType(archive, path, type) {
        return __awaiter(this, void 0, void 0, function* () {
            return XmlHelper.getRelationships(archive, path, (element, rels) => {
                let target = element.getAttribute('Type');
                if (target === type) {
                    rels.push({
                        file: element.getAttribute('Target'),
                        rId: element.getAttribute('Id'),
                    });
                }
            });
        });
    }
    static getRelationships(archive, path, cb) {
        return __awaiter(this, void 0, void 0, function* () {
            let xml = yield XmlHelper.getXmlFromArchive(archive, path);
            let relationships = xml.getElementsByTagName('Relationship');
            let rels = [];
            for (let i in relationships) {
                let element = relationships[i];
                if (element.getAttribute !== undefined) {
                    cb(element, rels);
                }
            }
            return rels;
        });
    }
    static findByAttribute(xml, tagName, attributeName, attributeValue) {
        let elements = xml.getElementsByTagName(tagName);
        for (let i in elements) {
            let element = elements[i];
            if (element.getAttribute !== undefined) {
                if (element.getAttribute(attributeName) === attributeValue) {
                    return true;
                }
            }
        }
        return false;
    }
    static replaceAttribute(archive, path, tagName, attributeName, attributeValue, replaceValue) {
        return __awaiter(this, void 0, void 0, function* () {
            let xml = yield XmlHelper.getXmlFromArchive(archive, path);
            let elements = xml.getElementsByTagName(tagName);
            for (let i in elements) {
                let element = elements[i];
                if (element.getAttribute !== undefined && element.getAttribute(attributeName) === attributeValue) {
                    element.setAttribute(attributeName, replaceValue);
                }
            }
            return XmlHelper.writeXmlToArchive(archive, path, xml);
        });
    }
    static getTargetByRelId(archive, slideNumber, element, type) {
        return __awaiter(this, void 0, void 0, function* () {
            let params = constants_1.TargetByRelIdMap[type];
            let sourceRid = element.getElementsByTagName(params.relRootTag)[0].getAttribute(params.relAttribute);
            let relsPath = `ppt/slides/_rels/slide${slideNumber}.xml.rels`;
            let imageRels = yield XmlHelper.getTargetsFromRelationships(archive, relsPath, params.prefix, params.expression);
            let target = imageRels.find(rel => rel.rId === sourceRid);
            return target;
        });
    }
    static findByElementName(archive, path, name) {
        return __awaiter(this, void 0, void 0, function* () {
            let slideXml = yield XmlHelper.getXmlFromArchive(archive, path);
            return XmlHelper.findByName(slideXml, name);
        });
    }
    static findByName(doc, name) {
        let names = doc.getElementsByTagName('p:cNvPr');
        for (let i in names) {
            if (names[i].getAttribute && names[i].getAttribute('name') === name) {
                return names[i].parentNode.parentNode;
            }
        }
        return null;
    }
    static findByAttributeValue(nodes, attributeName, attributeValue) {
        for (let i in nodes) {
            if (nodes[i].getAttribute && nodes[i].getAttribute(attributeName) === attributeValue) {
                return nodes[i];
            }
        }
        return null;
    }
    static createContentTypeChild(archive, attributes) {
        return {
            archive: archive,
            file: `[Content_Types].xml`,
            parent: (xml) => xml.getElementsByTagName('Types')[0],
            tag: 'Override',
            attributes: attributes
        };
    }
    static createRelationshipChild(archive, targetRelFile, attributes) {
        return {
            archive: archive,
            file: targetRelFile,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes: attributes
        };
    }
    static setChartData(chart, workbook, data) {
        let series = chart.getElementsByTagName('c:ser');
        for (let c in data.categories) {
            for (let s in data.categories[c].values) {
                series[s].getElementsByTagName('c:cat')[0]
                    .getElementsByTagName('c:v')[c]
                    .firstChild.data = data.categories[c].label;
                series[s].getElementsByTagName('c:v')[0]
                    .firstChild.data = data.series[s].label;
                series[s].getElementsByTagName('c:val')[0]
                    .getElementsByTagName('c:v')[c]
                    .firstChild.data = String(data.categories[c].values[s]);
            }
        }
        XmlHelper.setWorkbookData(workbook, data);
    }
    static setWorkbookData(workbook, data) {
        let rows = workbook.sheet.getElementsByTagName('row');
        for (let c in data.categories) {
            let r = Number(c) + 1;
            let stringId = XmlHelper.appendSharedString(workbook.sharedStrings, data.categories[c].label);
            let rowLabel = rows[r].getElementsByTagName('c')[0].getElementsByTagName('v')[0];
            rowLabel.firstChild.data = String(stringId);
            for (let s in data.categories[c].values) {
                let v = Number(s) + 1;
                rows[r].getElementsByTagName('c')[v]
                    .getElementsByTagName('v')[0]
                    .firstChild.data = String(data.categories[c].values[s]);
            }
        }
        for (let s in data.series) {
            let c = Number(s) + 1;
            let colLabel = rows[0].getElementsByTagName('c')[c].getElementsByTagName('v')[0];
            let stringId = XmlHelper.appendSharedString(workbook.sharedStrings, data.series[s].label);
            colLabel.firstChild.data = String(stringId);
            workbook.table.getElementsByTagName('tableColumn')[c].setAttribute('name', data.series[s].label);
        }
    }
    static appendSharedString(sharedStrings, string) {
        let strings = sharedStrings.getElementsByTagName('sst')[0];
        let newLabel = sharedStrings.createTextNode(string);
        let newText = sharedStrings.createElement('t');
        newText.appendChild(newLabel);
        let newString = sharedStrings.createElement('si');
        newString.appendChild(newText);
        strings.appendChild(newString);
        let stringId = strings.getElementsByTagName('si').length - 1;
        return stringId;
    }
    static dump(element) {
        let s = new xmldom_1.XMLSerializer();
        let xmlBuffer = s.serializeToString(element);
        let p = new pretty_1.default(xmlBuffer);
        p.dump();
    }
}
exports.default = XmlHelper;
//# sourceMappingURL=xml.js.map