"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const file_1 = __importDefault(require("./helper/file"));
const xml_1 = __importDefault(require("./helper/xml"));
const shape_1 = __importDefault(require("./shape"));
class Image extends shape_1.default {
    constructor(relsXmlInfo, sourceArchive, sourceSlideNumber) {
        super(relsXmlInfo, sourceArchive, sourceSlideNumber);
        this.sourceFile = relsXmlInfo.file.replace('../media/', '');
        this.extension = file_1.default.getFileExtension(this.sourceFile);
        let mapRelRootTags = {
            svg: 'asvg:svgBlip'
        };
        this.relRootTag = (mapRelRootTags[this.extension])
            ? mapRelRootTags[this.extension]
            : 'a:blip';
        this.relAttribute = 'r:embed';
        this.relParent = element => element.parentNode.parentNode;
        this.contentTypeMap = {
            jpg: "image/jpeg",
            jpeg: "image/jpeg",
            png: "image/png",
            gif: "image/gif",
            svg: "image/svg+xml",
            m4v: "video/mp4",
            mp4: "video/mp4",
            emf: "image/x-emf"
        };
    }
    append(targetTemplate, targetSlideNumber, appendToTree) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setTarget(targetTemplate, targetSlideNumber);
            this.targetNumber = this.targetTemplate.incrementCounter('images');
            this.targetFile = 'image' + this.targetNumber + '.' + this.extension;
            yield this.copyFiles();
            yield this.appendTypes();
            yield this.appendToSlideRels();
            if (appendToTree) {
                yield this.appendToSlideTree();
            }
            yield this.updateElementRelId();
            return this;
        });
    }
    copyFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            file_1.default.zipCopy(this.sourceArchive, `ppt/media/${this.sourceFile}`, this.targetArchive, `ppt/media/${this.targetFile}`);
        });
    }
    appendTypes() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.appendImageExtensionToContentType();
        });
    }
    appendToSlideRels() {
        return __awaiter(this, void 0, void 0, function* () {
            let targetRelFile = `ppt/slides/_rels/slide${this.targetSlideNumber}.xml.rels`;
            this.createdRid = yield xml_1.default.getNextRelId(this.targetArchive, targetRelFile);
            let attributes = {
                Id: this.createdRid,
                Type: "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image",
                Target: `../media/image${this.targetNumber}.${this.extension}`
            };
            return xml_1.default.append(xml_1.default.createRelationshipChild(this.targetArchive, targetRelFile, attributes));
        });
    }
    appendImageExtensionToContentType() {
        let extension = this.extension;
        let contentType = (this.contentTypeMap[extension]) ? this.contentTypeMap[extension] : 'image/' + extension;
        return xml_1.default.appendIf(Object.assign(Object.assign({}, xml_1.default.createContentTypeChild(this.targetArchive, {
            Extension: extension,
            ContentType: contentType
        })), { tag: 'Default', clause: (xml) => !xml_1.default.findByAttribute(xml, 'Default', 'Extension', extension) }));
    }
    static getAllOnSlide(archive, relsPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield xml_1.default.getTargetsByRelationshipType(archive, relsPath, "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image");
        });
    }
}
exports.default = Image;
//# sourceMappingURL=image.js.map