"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xml_1 = __importDefault(require("./helper/xml"));
const general_1 = __importDefault(require("./helper/general"));
class Shape {
    constructor(shape) {
        this.mode = shape.mode;
        this.name = shape.name;
        this.sourceArchive = shape.sourceArchive;
        this.sourceSlideNumber = shape.sourceSlideNumber;
        this.sourceSlideFile = `ppt/slides/slide${this.sourceSlideNumber}.xml`;
        this.sourceElement = shape.sourceElement;
        this.callbacks = general_1.default.arrayify(shape.callback);
        if (shape.target) {
            this.sourceNumber = shape.target.number;
            this.sourceRid = shape.target.rId;
        }
    }
    setTarget(targetTemplate, targetSlideNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetTemplate = targetTemplate;
            this.targetArchive = yield this.targetTemplate.archive;
            this.targetSlideNumber = targetSlideNumber;
            this.targetSlideFile = `ppt/slides/slide${this.targetSlideNumber}.xml`;
            this.targetSlideRelFile = `ppt/slides/_rels/slide${this.targetSlideNumber}.xml.rels`;
        });
    }
    setTargetElement() {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetElement = this.sourceElement.cloneNode(true);
        });
    }
    appendToSlideTree() {
        return __awaiter(this, void 0, void 0, function* () {
            let targetSlideXml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetSlideFile);
            targetSlideXml.getElementsByTagName('p:spTree')[0].appendChild(this.targetElement);
            yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetSlideFile, targetSlideXml);
        });
    }
    replaceIntoSlideTree() {
        return __awaiter(this, void 0, void 0, function* () {
            let targetSlideXml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetSlideFile);
            let sourceElementOnTargetSlide = yield xml_1.default.findByName(targetSlideXml, this.name);
            sourceElementOnTargetSlide.parentNode.insertBefore(this.targetElement, sourceElementOnTargetSlide);
            sourceElementOnTargetSlide.parentNode.removeChild(sourceElementOnTargetSlide);
            yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetSlideFile, targetSlideXml);
        });
    }
    updateElementRelId() {
        return __awaiter(this, void 0, void 0, function* () {
            let targetSlideXml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetSlideFile);
            let targetElement = yield this.getElementByRid(targetSlideXml, this.sourceRid);
            targetElement.getElementsByTagName(this.relRootTag)[0].setAttribute(this.relAttribute, this.createdRid);
            yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetSlideFile, targetSlideXml);
        });
    }
    updateTargetElementRelId() {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetElement.getElementsByTagName(this.relRootTag)[0].setAttribute(this.relAttribute, this.createdRid);
        });
    }
    getElementByRid(slideXml, rId) {
        return __awaiter(this, void 0, void 0, function* () {
            let sourceList = slideXml.getElementsByTagName('p:spTree')[0].getElementsByTagName(this.relRootTag);
            let sourceElement = xml_1.default.findByAttributeValue(sourceList, this.relAttribute, rId);
            return this.relParent(sourceElement);
        });
    }
    applyCallbacks(callbacks, element, arg1, arg2) {
        for (let i in callbacks) {
            callbacks[i](element, arg1, arg2);
        }
    }
}
exports.default = Shape;
//# sourceMappingURL=shape.js.map